//------------------------------------------------------------------------------
//                    Delphi JBS Typed Controls package
//                               Version 2.0
//------------------------------------------------------------------------------
// This unit contents typed control which allows You to assign CheckBox to some
// property of another component. Property Type has to be Boolean.
// So changing of CheckBox's value calls the assigned property's
// value changing.
//
// Contents:   TTypedCheckBox component
//
// Writen by  Jack Bars  {October 1998}
//
//------------------------------------------------------------------------------
//                    !!! SEE BELOW FOR DETAILS !!!
//------------------------------------------------------------------------------

unit TypedCheckBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, TypInfo, ExtCtrls;

type

//------------------------------------------------------------------------------
// TTypedCheckBox component
//------------------------------------------------------------------------------
// This one allows You to assign itself to a property of another component which
// is in the same form. It can be assigned ONLY to a property which tipe is
// Boolean.
//
// Put TTypedCheckBox on the form. Assign it to the component of the same form,
// and then to one of that component's boolean properties.
//
// Properties
//
//       Component: Choose the component to assign from the dropdown value list.
//            Right after changin' value of this property, value of PropertyName
//            property becomes an empty string.
//       PropertyName: Determines the name of an assigned property. Choose value
//            of this one from the value list. It allows You to click on the
//            TypedCheckBox RunTime and it automatically calls the assigned
//            property's value changing.
//------------------------------------------------------------------------------

  TTypedCheckBox = class(TCustomCheckBox)
  private
    FPropertyName: string;
    FComponent: TComponent;
    { Private declarations }
  protected
    procedure Click; override;
    procedure SetComponent(NewComponent: TComponent);
    procedure SetPropertyName(NewPropertyName: string);
    { Protected declarations }
  public
    Timer: TTimer;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function PropInfo: PPropInfo;
    procedure CheckRefresh;
    constructor Create(AOwner: TComponent); override;
    procedure Loaded; override;
    destructor Destroy; override;
    procedure TimerEvent(Sender: TObject);
    { Public declarations }
  published
    property Component: TComponent read FComponent write SetComponent;
    property PropertyName: string read FPropertyName write SetPropertyName;
    property Alignment;
    property Caption;
    property Checked;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    { Published declarations }
  end;


implementation


procedure TTypedCheckBox.TimerEvent(Sender: TObject);
begin
  Timer.Enabled := false;
  CheckRefresh;
  Timer.Enabled := true;
end;


constructor TTypedCheckBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  AllowGrayed := false;
end;


procedure TTypedCheckBox.Loaded;
begin
  inherited Loaded;
  Timer := TTimer.Create(nil);
  Timer.Interval := 110;
  Timer.OnTimer := TimerEvent;
  Timer.Enabled := True;
end;


destructor TTypedCheckBox.Destroy;
begin
  Timer.Free;
  inherited Destroy;
end;


procedure TTypedCheckBox.Notification(AComponent: TComponent;
                                                        Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (Component <> nil)
  and not(csDestroying in ComponentState) then
    if AComponent = Component then
      Component := nil;
end;


procedure TTypedCheckBox.CheckRefresh;
begin
  if (Component <> nil) and (PropertyName <> '') then
  begin
    if GetOrdProp(Component, PropInfo) = 0 then
      Checked := False
    else
      Checked := True;
  end;
end;


function TTypedCheckBox.PropInfo: PPropInfo;
var
  i, Count: Word;
  PropList: PPropList;
begin
  if (Component = nil) or (PropertyName = '') then
  begin
    Result := nil;
    Exit;
  end;
  Count := GetPropList(Component.ClassInfo , [tkEnumeration], nil);
  GetMem(PropList, Count * SizeOf(PPropInfo));
  try
    GetPropList(Component.ClassInfo, [tkEnumeration], PropList);
    for i := 0 to Count - 1 do
      if (PropList^[i])^.Name = PropertyName then
      begin
        Result := PropList^[i];
      end;
  finally
    FreeMem(PropList, Count * SizeOf(PPropInfo));
  end;
end;


procedure TTypedCheckBox.Click;
var
  Value: Longint;
begin
  if (csLoading in ComponentState) or (csReading in ComponentState) then Exit;
  if (Component <> nil) and (PropertyName <> '') then
  begin
    Value := LongInt(Checked);
    SetOrdProp(Component, PropInfo, Value);
  end;
  inherited Click;
end;


procedure TTypedCheckBox.SetComponent(NewComponent: TComponent);
begin
  if FComponent <> NewComponent then
  begin
    FComponent := NewComponent;
    if not (csReading in ComponentState) and not (csLoading in ComponentState) then
    begin
      PropertyName := '';
    end;
  end;
end;


procedure TTypedCheckBox.SetPropertyName(NewPropertyName: string);
begin
  if FPropertyName <> NewPropertyName then
  begin
    FPropertyName := NewPropertyName;
  end;
end;


end.
