//------------------------------------------------------------------------------
//                    Delphi JBS Typed Controls package
//                               Version 2.0
//------------------------------------------------------------------------------
// This unit contents typed control which allows You to assign CheckListBox to
// some property of another component. Property Type has to be [tkSet].
// So changing of CheckListBox's value calls the assigned property's value
// changing.
//
// Contents:   TTypedCheckListBox component
//
// Writen by  Jack Bars  {October 1998}
//
//------------------------------------------------------------------------------
//                    !!! SEE BELOW FOR DETAILS !!!
//------------------------------------------------------------------------------

unit TypedCheckListBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, checklst, TypInfo, Math, extctrls;

type


//------------------------------------------------------------------------------
// TTypedCheckListBox component
//------------------------------------------------------------------------------
// This one allows You to assign itself to a property of another component which
// is in the same form. It can be assigned ONLY to a property which type is
// [tkSet].
//
// Put TTypedCheckList on the form. Assign it to the component of the same form
// and then to one of that component's properties. Type of property has to be
// [tkSet]
//
// Properties
//
//       Component: Choose the component to assign from the dropdown value list.
//            Right after changin' value of this property, value of PropertyName
//            property becomes an empty string.
//       PropertyName: Determines the name of an assigned property. Choose value
//            of this one from the value list. This automaticaly calls adding of
//            Items property with all possible values of choosen set property.
//            So that allows You to check those values RunTime in the
//            TypedCheckListBox and it automatically calls the assigned
//            property's value changing.
//------------------------------------------------------------------------------

  TTypedCheckListBox = class(TCheckListBox)
  private
    FPropertyName: string;
    FComponent: TComponent;
    { Private declarations }
  protected
    procedure Click; override;
    procedure SetComponent(NewComponent: TComponent);
    procedure SetPropertyName(NewPropertyName: string);
    { Protected declarations }
  public
    Timer: TTimer;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function PropInfo: PPropInfo;
    procedure Loaded; override;
    destructor Destroy; override;
    procedure CheckRefresh;
    procedure TimerEvent(Sender: TObject);
    { Public declarations }
  published
    property Component: TComponent read FComponent write SetComponent;
    property PropertyName: string read FPropertyName write SetPropertyName;
    { Published declarations }
  end;


implementation


procedure TTypedCheckListBox.Notification(AComponent: TComponent;
                                                        Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (Component <> nil)
  and not(csDestroying in ComponentState) then
    if AComponent = Component then
      Component := nil;
end;


procedure TTypedCheckListBox.TimerEvent(Sender: TObject);
begin
  Timer.Enabled := false;
  CheckRefresh;
  Timer.Enabled := true;
end;


procedure TTypedCheckListBox.Loaded;
begin
  inherited Loaded;
  Timer := TTimer.Create(nil);
  Timer.Interval := 550;
  Timer.OnTimer := TimerEvent;
  Timer.Enabled := True;
end;


destructor TTypedCheckListBox.Destroy;
begin
  Timer.Free;
  inherited Destroy;
end;


procedure TTypedCheckListBox.Click;
var
  Value: Cardinal;
  i,k, bufValue: Cardinal;
begin
  if (csLoading in ComponentState) or (csReading in ComponentState) then Exit;
  if (Component <> nil) and (PropertyName <> '') then
  begin
    Timer.Enabled := false;
    Value := 0;
    for i := 0 to Items.Count - 1 do
      if Checked[i] then
      begin
        Value := Value + Trunc(Power(2, i));
      end;
    SetOrdProp(Component, PropInfo, Value);
  end;
  inherited Click;
  Timer.Enabled := true;
end;


function TTypedCheckListBox.PropInfo: PPropInfo;
var
  i, Count: Word;
  PropList: PPropList;
begin
  if (Component = nil) or (PropertyName = '') then
  begin
    Result := nil;
    Exit;
  end;
  Count := GetPropList(Component.ClassInfo , [tkSet], nil);
  GetMem(PropList, Count * SizeOf(PPropInfo));
  try
    GetPropList(Component.ClassInfo, [tkSet], PropList);
    for i := 0 to Count - 1 do
      if (PropList^[i])^.Name = PropertyName then
      begin
        Result := PropList^[i];
      end;
  finally
    FreeMem(PropList, Count * SizeOf(PPropInfo));
  end;
end;


procedure TTypedCheckListBox.CheckRefresh;
var
  Value: Cardinal;
  i: Word;
begin
  if (Component <> nil) and (PropertyName <> '') then
  begin
    Value := GetOrdProp(Component, PropInfo);
    for i := Items.Count -1 downto 0 do
      if ( Value - Trunc(Power(2, i)) ) >= 0 then
      begin
        Value := Value - Trunc(Power(2, i));
        Checked[i] := True;
      end
      else
        Checked[i] := false;
  end;
end;


procedure TTypedCheckListBox.SetComponent(NewComponent: TComponent);
begin
  if FComponent <> NewComponent then
  begin
    FComponent := NewComponent;
    if not (csReading in ComponentState) and not (csLoading in ComponentState) then
    begin
      PropertyName := '';
      Clear;
    end;
  end;
end;


procedure TTypedCheckListBox.SetPropertyName(NewPropertyName: string);
var
  Value: LongInt;
begin
  if FPropertyName <> NewPropertyName then
  begin
    FPropertyName := NewPropertyName;
    if not (csReading in ComponentState)
    and not (csLoading in ComponentState) then
    begin
      Clear;
      if (FPropertyName <> '') and (PropInfo <> nil) then
      begin
        for Value := GetTypeData( (GetTypeData(PropInfo^.PropType^))^.CompType^ )^.MinValue  to
        GetTypeData( (GetTypeData(PropInfo^.PropType^))^.CompType^ )^.MaxValue do
          Items.Add(GetEnumName( (GetTypeData(PropInfo^.PropType^))^.CompType^ , Value));
        CheckRefresh;  
      end;
    end;
  end;
end;


end.
