//------------------------------------------------------------------------------
//                    Delphi JBS Typed Controls package
//                               Version 2.0
//------------------------------------------------------------------------------
// This unit contents typed control which allows You to assign ComboBox to some
// enumerated property of another component. So changing of ComboBox's value
// calls the assigned property's value changing.
//
// Contents:   TTypedComboBox component
//
// Writen by  Jack Bars  {October 1998}
//
//------------------------------------------------------------------------------
//                    !!! SEE BELOW FOR DETAILS !!!
//------------------------------------------------------------------------------

unit TypedComboBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, TypInfo, ExtCtrls;

type

//------------------------------------------------------------------------------
// TTypedComboBox component
//------------------------------------------------------------------------------
// This one allows You to assign itself to a property of another component which
// is in the same form. It can be assigned ONLY to a property which type is
// [tkEnumeration].
//
// Put TTypedComboBox on the form. Assign it to the component of the same form,
// and then to one of that component's enumerated properties.
//
// Properties
//
//       Component: Choose the component to assign from the dropdown value list.
//            Right after changin' value of this property, value of PropertyName
//            property becomes an empty string.
//       PropertyName: Determines the name of an assigned property. Choose value
//            of this one from the value list. This automaticaly calls adding of
//            Items property with all possible values of choosen enumerated
//            property. So that allows You to choose one of those values RunTime
//            from TypedComboBox and it automatically calls the assigned
//            property's value changing.
//------------------------------------------------------------------------------

  TTypedComboBox = class(TCustomComboBox)
  private
    FPropertyName: string;
    FComponent: TComponent;
    { Private declarations }
  protected
    procedure Change; override;
    procedure SetComponent(NewComponent: TComponent);
    procedure SetPropertyName(NewPropertyName: string);
    { Protected declarations }
  public
    Timer: TTimer;
    destructor Destroy; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function PropInfo: PPropInfo;
    procedure Loaded; override;
    procedure TextRefresh;
    procedure TimerEvent(Sender: TObject);
    { Public declarations }
  published
    property Component: TComponent read FComponent write SetComponent;
    property PropertyName: string read FPropertyName write SetPropertyName;
    property Style;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property DropDownCount;
    property Enabled;
    property Font;
    property ImeMode;
    property ImeName;
    property ItemHeight;
    property Items;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMeasureItem;
    property OnStartDrag;
    { Published declarations }
  end;


implementation


procedure TTypedComboBox.TimerEvent(Sender: TObject);
begin
  Timer.Enabled := false;
  TextRefresh;
  Timer.Enabled := true;
end;


procedure TTypedComboBox.Loaded;
begin
  inherited Loaded;
  Timer := TTimer.Create(nil);
  Timer.Interval := 550;
  Timer.OnTimer := TimerEvent;
  Timer.Enabled := True;
end;


destructor TTypedComboBox.Destroy;
begin
  Timer.Free;
  inherited Destroy;
end;



procedure TTypedComboBox.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (Component <> nil) and not(csDestroying in ComponentState) then
    if AComponent = Component then
      Component := nil;
end;


procedure TTypedComboBox.TextRefresh;
var
  Value: LongInt;
begin
  if (Component <> nil) and (PropertyName <> '') then
  begin
    Value := GetOrdProp(Component, PropInfo);
    Text := GetEnumName(PropInfo^.PropType^, Value);
  end;
end;


procedure TTypedComboBox.Change;
var
  Value: Longint;
begin
  inherited Change;
  if (csLoading in ComponentState) or (csReading in ComponentState) then Exit;
  if (Component <> nil) and (PropertyName <> '') and (Text <> '') then
  begin
    Value := GetEnumValue(PropInfo^.PropType^, Text);
    SetOrdProp(Component, PropInfo, Value);
  end;
end;


procedure TTypedComboBox.SetComponent(newComponent: TComponent);
begin
  if FComponent <> NewComponent then
  begin
    FComponent := NewComponent;
    if not (csReading in ComponentState) and not (csLoading in ComponentState) then
    begin
      PropertyName := '';
      Clear;
    end;
  end;
end;


function TTypedComboBox.PropInfo: PPropInfo;
var
  i, Count: Word;
  PropList: PPropList;
begin
  if (Component = nil) or (PropertyName = '') then
  begin
    Result := nil;
    Exit;
  end;
  Count := GetPropList(Component.ClassInfo , [tkEnumeration], nil);
  GetMem(PropList, Count * SizeOf(PPropInfo));
  try
    GetPropList(Component.ClassInfo, [tkEnumeration], PropList);
    for i := 0 to Count - 1 do
      if (PropList^[i])^.Name = PropertyName then
      begin
        Result := PropList^[i];
      end;
  finally
    FreeMem(PropList, Count * SizeOf(PPropInfo));
  end;
end;


procedure TTypedComboBox.SetPropertyName(NewPropertyName: string);
var
  Value: LongInt;
begin
  if FPropertyName <> NewPropertyName then
  begin
    FPropertyName := NewPropertyName;
    if not (csReading in ComponentState) and not (csLoading in ComponentState) then
    begin
      Clear;
      if (FPropertyName <> '') and (PropInfo <> nil) then
      begin
        for Value := (GetTypeData(PropInfo^.PropType^))^.MinValue to (GetTypeData(PropInfo^.PropType^))^.MaxValue do
          Items.Add(GetEnumName(PropInfo^.PropType^, Value));
      end;
    end;
  end;
end;


end.
