//------------------------------------------------------------------------------
//                    Delphi JBS Typed Controls package
//                               Version 2.0
//------------------------------------------------------------------------------
// This unit contents typed control which allows You to assign Edit to some
// property of another component. Property Type has to be [tkString], [WString]
// or [LString].  So changing of Edit's value calls the assigned property's
// value changing.
//
// Contents:   TTypedEdit component
//
// Writen by  Jack Bars  {October 1998}
//
//------------------------------------------------------------------------------
//                    !!! SEE BELOW FOR DETAILS !!!
//------------------------------------------------------------------------------

unit TypedEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, TypInfo, ExtCtrls;

type

//------------------------------------------------------------------------------
// TTypedEdit component
//------------------------------------------------------------------------------
// This one allows You to assign itself to a property of another component which
// is in the same form. It can be assigned ONLY to a property which tipe is
// [tkString], [tkLString] or [tkWString].
//
// Put TTypedEdit on the form. Assign it to the component of the same form,
// and then to one of that component's string_type properties.
//
// Properties
//
//       Component: Choose the component to assign from the dropdown value list.
//            Right after changin' value of this property, value of PropertyName
//            property becomes an empty string.
//       PropertyName: Determines the name of an assigned property. Choose value
//            of this one from the value list. It allows You to enter the Text
//            in the TypedEdit and it automatically calls the assigned
//            property's value changing.
//------------------------------------------------------------------------------

  TTypedEdit = class(TCustomEdit)
  private
    FPropertyName: string;
    FComponent: TComponent;
    { Private declarations }
  protected
    procedure Change; override;
    procedure SetComponent(NewComponent: TComponent);
    procedure SetPropertyName(NewPropertyName: string);
    { Protected declarations }
  public
    Timer: TTimer;
    procedure Loaded; override;
    destructor Destroy; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function PropInfo: PPropInfo;
    procedure TextRefresh;
    procedure TimerEvent(Sender: TObject);
    { Public declarations }
  published
    property Component: TComponent read FComponent write SetComponent;
    property PropertyName: string read FPropertyName write SetPropertyName;
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property ImeMode;
    property ImeName;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PasswordChar;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Text;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
    { Published declarations }
  end;


implementation


procedure TTypedEdit.TimerEvent(Sender: TObject);
begin
  Timer.Enabled := false;
  TextRefresh;
  Timer.Enabled := true;
end;


procedure TTypedEdit.Loaded;
begin
  inherited Loaded;
  Timer := TTimer.Create(nil);
  Timer.Interval := 550;
  Timer.OnTimer := TimerEvent;
  Timer.Enabled := True;
end;


destructor TTypedEdit.Destroy;
begin
  Timer.Free;
  inherited Destroy;
end;

procedure TTypedEdit.Change;
begin
  inherited Change;
  if (csLoading in ComponentState) or (csReading in ComponentState) then Exit;
  if (Component <> nil) and (PropertyName <> '') and (Text <> '') then
  begin
    SetStrProp(Component, PropInfo, Text);
  end;
end;


procedure TTypedEdit.Notification(AComponent: TComponent;
                                                        Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (Component <> nil)
  and not(csDestroying in ComponentState) then
    if AComponent = Component then
      Component := nil;
end;


function TTypedEdit.PropInfo: PPropInfo;
var
  i, Count: Word;
  PropList: PPropList;
begin
  if (Component = nil) or (PropertyName = '') then
  begin
    Result := nil;
    Exit;
  end;
  Count := GetPropList(Component.ClassInfo , [tkString, tkWString, tkLString], nil);
  GetMem(PropList, Count * SizeOf(PPropInfo));
  try
    GetPropList(Component.ClassInfo, [tkString, tkWString, tkLString], PropList);
    for i := 0 to Count - 1 do
      if (PropList^[i])^.Name = PropertyName then
      begin
        Result := PropList^[i];
      end;
  finally
    FreeMem(PropList, Count * SizeOf(PPropInfo));
  end;
end;


procedure TTypedEdit.TextRefresh;
begin
  if (Component <> nil) and (PropertyName <> '') then
  begin
    Text := GetStrProp(Component, PropInfo);
  end;
end;


procedure TTypedEdit.SetComponent(NewComponent: TComponent);
begin
  if FComponent <> NewComponent then
  begin
    FComponent := NewComponent;
    if not (csReading in ComponentState) and not (csLoading in ComponentState) then
    begin
      PropertyName := '';
      Text := '';
    end;
  end;
end;


procedure TTypedEdit.SetPropertyName(NewPropertyName: string);
begin
  if FPropertyName <> NewPropertyName then
  begin
    FPropertyName := NewPropertyName;
    TextRefresh;
  end;
end;


end.
