//------------------------------------------------------------------------------
//                    Delphi JBS Typed Controls package
//                               Version 2.0
//------------------------------------------------------------------------------
// This unit contents typed control which allows You to assign RadioGroup to
// some property of another component. Property Type has to be [tkEnumeration].
// So changing of RadioGroup's value calls the assigned property's value
// changing.
//
// Contents:   TTypedRadioGroup component
//
// Writen by  Jack Bars  {October 1998}
//
//------------------------------------------------------------------------------
//                    !!! SEE BELOW FOR DETAILS !!!
//------------------------------------------------------------------------------

unit TypedRadioGroup;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, TypInfo;

type

//------------------------------------------------------------------------------
// TTypedRadioGroup component
//------------------------------------------------------------------------------
// This one allows You to assign itself to a property of another component which
// is in the same form. It can be assigned ONLY to a property which type is
// [tkEnumeration].
//
// Put TTypedRadioGroup on the form. Assign it to the component of the same form
// and then to one of that component's enumerated properties.
//
// Properties
//
//       Component: Choose the component to assign from the dropdown value list.
//            Right after changin' value of this property, value of PropertyName
//            property becomes an empty string.
//       PropertyName: Determines the name of an assigned property. Choose value
//            of this one from the value list. This automaticaly calls adding of
//            Items property with all possible values of choosen enumerated
//            property. So that allows You to choose one of those values RunTime
//            from TypedRadioGroup and it automatically calls the assigned
//            property's value changing.
//------------------------------------------------------------------------------

  TTypedRadioGroup = class(TCustomRadioGroup)
  private
    FPropertyName: string;
    FComponent: TComponent;
    { Private declarations }
  protected
    procedure Click; override;
    procedure SetComponent(NewComponent: TComponent);
    procedure SetPropertyName(NewPropertyName: string);
    { Protected declarations }
  public
    Timer: TTimer;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    destructor Destroy; override;
    function PropInfo: PPropInfo;
    procedure Loaded; override;
    procedure CheckRefresh;
    procedure TimerEvent(Sender: TObject);
    { Public declarations }
  published
    property Component: TComponent read FComponent write SetComponent;
    property PropertyName: string read FPropertyName write SetPropertyName;
    property Align;
    property Caption;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Items;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnStartDrag;
    { Published declarations }
  end;


implementation


procedure TTypedRadioGroup.TimerEvent(Sender: TObject);
begin
  Timer.Enabled := false;
  CheckRefresh;
  Timer.Enabled := true;
end;


procedure TTypedRadioGroup.Loaded;
begin
  inherited Loaded;
  Timer := TTimer.Create(nil);
  Timer.Interval := 550;
  Timer.OnTimer := TimerEvent;
  Timer.Enabled := True;
end;


destructor TTypedRadioGroup.Destroy;
begin
  Timer.Free;
  inherited Destroy;
end;

procedure TTypedRadioGroup.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (Component <> nil) and not(csDestroying in ComponentState) then
    if AComponent = Component then
      Component := nil;
end;


procedure TTypedRadioGroup.CheckRefresh;
begin
  if (Component <> nil) and (PropertyName <> '') then
  try
    ItemIndex := GetOrdProp(Component, PropInfo);
//    Text := GetEnumName(PropInfo^.PropType^, Value);
  except
  end;
end;


procedure TTypedRadioGroup.Click;
begin
  inherited Click;
  if (csLoading in ComponentState) or (csReading in ComponentState) then Exit;
  if (Component <> nil) and (PropertyName <> '') and (ItemIndex <>  -1) then
  begin
    SetOrdProp(Component, PropInfo, ItemIndex);
  end;
end;


procedure TTypedRadioGroup.SetComponent(newComponent: TComponent);
begin
  if FComponent <> NewComponent then
  begin
    FComponent := NewComponent;
    if not (csReading in ComponentState) and not (csLoading in ComponentState) then
    begin
      PropertyName := '';
      Items.Clear;
    end;
  end;
end;


function TTypedRadioGroup.PropInfo: PPropInfo;
var
  i, Count: Word;
  PropList: PPropList;
begin
  if (Component = nil) or (PropertyName = '') then
  begin
    Result := nil;
    Exit;
  end;
  Count := GetPropList(Component.ClassInfo , [tkEnumeration], nil);
  GetMem(PropList, Count * SizeOf(PPropInfo));
  try
    GetPropList(Component.ClassInfo, [tkEnumeration], PropList);
    for i := 0 to Count - 1 do
      if (PropList^[i])^.Name = PropertyName then
      begin
        Result := PropList^[i];
      end;
  finally
    FreeMem(PropList, Count * SizeOf(PPropInfo));
  end;
end;


procedure TTypedRadioGroup.SetPropertyName(NewPropertyName: string);
var
  Value: LongInt;
begin
  if FPropertyName <> NewPropertyName then
  begin
    FPropertyName := NewPropertyName;
    if not (csReading in ComponentState) and not (csLoading in ComponentState) then
    begin
      Items.Clear;
      if (FPropertyName <> '') and (PropInfo <> nil) then
      begin
        for Value := (GetTypeData(PropInfo^.PropType^))^.MinValue to (GetTypeData(PropInfo^.PropType^))^.MaxValue do
          Items.Add(GetEnumName(PropInfo^.PropType^, Value));
      end;
    end;
  end;
end;


end.
