//------------------------------------------------------------------------------
//                    Delphi JBS Typed Controls package
//                               Version 2.0
//------------------------------------------------------------------------------
// This unit contents typed control which allows You to assign SpinEdit to some
// property of another component. Property Type has to be [tkInteger].
// So changing of SpinEdit's value calls the assigned property's value changing.
//
// Contents:   TTypedSpinEdit component
//
// Writen by  Jack Bars  {October 1998}
//
//------------------------------------------------------------------------------
//                    !!! SEE BELOW FOR DETAILS !!!
//------------------------------------------------------------------------------

unit TypedSpinEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, TypInfo, ExtCtrls;

type

//------------------------------------------------------------------------------
// TTypedEdit component
//------------------------------------------------------------------------------
// This one allows You to assign itself to a property of another component which
// is in the same form. It can be assigned to a property which tipe is
// [tkInteger] ONLY.
//
// Put TTypedSpinEdit on the form. Assign it to the component of the same form,
// and then to one of that component's [tkInteger] properties.
//
// Properties
//
//       Component: Choose the component to assign from the dropdown value list.
//            Right after changin' value of this property, value of PropertyName
//            property becomes an empty string.
//       PropertyName: Determines the name of an assigned property. Choose value
//            of this one from the value list. It allows You to enter the Value
//            in the TypedSpinEdit and it automatically calls the assigned
//            property's value changing.
//------------------------------------------------------------------------------

  TTypedSpinEdit = class(TSpinEdit)
  private
    FPropertyName: string;
    FComponent: TComponent;
    { Private declarations }
  protected
    procedure Change; override;
    procedure SetComponent(NewComponent: TComponent);
    procedure SetPropertyName(NewPropertyName: string);
    { Protected declarations }
  public
    Timer: TTimer;
    destructor Destroy; override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function PropInfo: PPropInfo;
    procedure ValueRefresh;
    procedure TimerEvent(Sender: TObject);
    { Public declarations }
  published
    property Component: TComponent read FComponent write SetComponent;
    property PropertyName: string read FPropertyName write SetPropertyName;
    { Published declarations }
  end;


implementation


procedure TTypedSpinEdit.TimerEvent(Sender: TObject);
begin
  Timer.Enabled := false;
  ValueRefresh;
  Timer.Enabled := true;
end;


procedure TTypedSpinEdit.Loaded;
begin
  inherited Loaded;
  Timer := TTimer.Create(nil);
  Timer.Interval := 110;
  Timer.OnTimer := TimerEvent;
  Timer.Enabled := True;
end;


destructor TTypedSpinEdit.Destroy;
begin
  Timer.Free;
  inherited Destroy;
end;



procedure TTypedSpinEdit.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (Component <> nil)
  and not(csDestroying in ComponentState) then
    if AComponent = Component then
      Component := nil;
end;


procedure TTypedSpinEdit.Change;
begin
  inherited Change;
  if (csLoading in ComponentState) or (csReading in ComponentState) then Exit;
  if (Component <> nil) and (PropertyName <> '') and (Text <> '')  then
  try
    SetOrdProp(Component, PropInfo, Value);
  finally
    ValueRefresh;
  end;
end;


function TTypedSpinEdit.PropInfo: PPropInfo;
var
  i, Count: Word;
  PropList: PPropList;
begin
  if (Component = nil) or (PropertyName = '') then
  begin
    Result := nil;
    Exit;
  end;
  Count := GetPropList(Component.ClassInfo , [tkInteger], nil);
  GetMem(PropList, Count * SizeOf(PPropInfo));
  try
    GetPropList(Component.ClassInfo, [tkInteger], PropList);
    for i := 0 to Count - 1 do
      if (PropList^[i])^.Name = PropertyName then
      begin
        Result := PropList^[i];
      end;
  finally
    FreeMem(PropList, Count * SizeOf(PPropInfo));
  end;
end;


procedure TTypedSpinEdit.ValueRefresh;
begin
  if (Component <> nil) and (PropertyName <> '') then
  try
    Value := GetOrdProp(Component, PropInfo);
  finally
  end;
end;


procedure TTypedSpinEdit.SetPropertyName(NewPropertyName: string);
begin
  if FPropertyName <> NewPropertyName then
  begin
    FPropertyName := NewPropertyName;
    ValueRefresh;
  end;
end;


procedure TTypedSpinEdit.SetComponent(NewComponent: TComponent);
begin
  if FComponent <> NewComponent then
  begin
    FComponent := NewComponent;
    if not (csReading in ComponentState) and not (csLoading in ComponentState) then
    begin
      PropertyName := '';
      Value := 0;
    end;
  end;
end;


end.
